using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.Forums;
using CMS.UIControls;
using CMS.CMSHelper;
using CMS.URLRewritingEngine;

public partial class CMSModules_Forums_Controls_Posts_PostEdit : CMSAdminEditControl
{
    #region "Variables"

    private int mForumID = 0;
    private int mEditPostID = 0;
    ForumInfo fi = null;
    ForumPostInfo editPi = null;
    ForumPostInfo replyPi = null;
    private bool mRedirectEnabled = true;

    #endregion


    #region "Public properties"

    /// <summary>
    /// Gets or sets the value that indicates whether redirect is enabled
    /// </summary>
    public bool RedirectEnabled
    {
        get
        {
            return mRedirectEnabled;
        }
        set
        {
            mRedirectEnabled = value;
        }
    }


    /// <summary>
    /// Gets or sets forum id
    /// </summary>
    public int ForumID
    {
        get
        {
            return mForumID;
        }
        set
        {
            mForumID = value;
        }
    }


    /// <summary>
    /// Gets or sets post id of reply post
    /// </summary>
    public int ReplyToPostID
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["ReplyToPostID"], 0);
        }
        set
        {
            ViewState["ReplyToPostID"] = value;
        }
    }


    /// <summary>
    /// Gets or sete post id of post which should be edited
    /// </summary>
    public int EditPostID
    {
        get
        {
            return mEditPostID;
        }
        set
        {
            mEditPostID = value;
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        bool process = true;
        if (!this.Visible || StopProcessing)
        {
            process = false;
        }

        fi = ForumInfoProvider.GetForumInfo(this.ForumID);
        replyPi = ForumPostInfoProvider.GetForumPostInfo(this.ReplyToPostID);
        editPi = ForumPostInfoProvider.GetForumPostInfo(this.EditPostID);

        if ((fi == null) && (editPi != null))
        {
            fi = ForumInfoProvider.GetForumInfo(editPi.PostForumID);
            this.ForumID = fi.ForumID;
        }

        #region "HTML Edtitor properties"

        // Set HTML editor properties
        htmlTemplateBody.AutoDetectLanguage = false;
        htmlTemplateBody.DefaultLanguage = System.Threading.Thread.CurrentThread.CurrentCulture.TwoLetterISOLanguageName;
        htmlTemplateBody.EditorAreaCSS = "";
        htmlTemplateBody.ToolbarSet = "Forum";
        htmlTemplateBody.IsLiveSite = this.IsLiveSite;

        #endregion

        // Regular expression to validate email (e-mail is not required)
        rfvEmail.ValidationExpression = @"^([\w0-9_\-\+]+(\.[\w0-9_\-\+]+)*@[\w0-9_-]+(\.[\w0-9_-]+)+)*$";

        if (fi != null)
        {

            if ((fi.ForumType == 0) && (replyPi == null))
            {
                plcThreadType.Visible = true;
            }

            if (fi.ForumRequireEmail)
            {
                rfvEmailRequired.Enabled = true;
                rfvEmailRequired.ErrorMessage = ResHelper.GetString("Forums_WebInterface_ForumNewPost.emailRequireErrorMsg");
            }

            #region "Forum text"

            rfvText.Enabled = !fi.ForumHTMLEditor;
            htmlTemplateBody.Visible = fi.ForumHTMLEditor;
            ucBBEditor.Visible = !fi.ForumHTMLEditor;

            if (fi.ForumHTMLEditor)
            {
                // Register script for HTML Editor forum buttons control
                ScriptHelper.RegisterClientScriptBlock(this, typeof(string), "InitForumFCKEditor", ScriptHelper.GetScript(
                    "function InitForumFCKEditor(editorId, showUrl, showImage, showQuote, showAdvUrl, showAdvImage) { \n" +
                        "var fckEditor = FCKeditorAPI.GetInstance(editorId); \n" +
                        "if (fckEditor != null) { \n" +
                            "var loadedItems = fckEditor.EditorWindow.parent.FCKToolbarItems.LoadedItems; \n" +
                            "if (!showUrl) { \n" +
                                "loadedItems['InsertUrl']._UIButton.MainElement.style.display = 'none'; \n" +
                            "} \n" +
                            "if (!showImage) { \n" +
                                "loadedItems['InsertImage']._UIButton.MainElement.style.display = 'none'; \n" +
                            "} \n" +
                            "if (!showQuote) {  \n" +
                                "loadedItems['InsertQuote']._UIButton.MainElement.style.display = 'none';  \n" +
                            "} \n" +
                            "if (!showAdvUrl) {  \n" +
                                "loadedItems['InsertLink']._UIButton.MainElement.style.display = 'none';  \n" +
                            "} \n" +
                            "if (!showAdvImage) {  \n" +
                                "loadedItems['InsertImageOrMedia']._UIButton.MainElement.style.display = 'none';  \n" +
                            "} \n" +
                        "} \n" +
                    "} \n"));

                this.ltlScript.Text += ScriptHelper.GetScript("function FCKeditor_OnComplete(editorInstance){ InitForumFCKEditor('" + this.htmlTemplateBody.ClientID + "', " + fi.ForumEnableURL.ToString().ToLower() + ", " + fi.ForumEnableImage.ToString().ToLower() + ", " + fi.ForumEnableQuote.ToString().ToLower() + ", " + fi.ForumEnableAdvancedURL.ToString().ToLower() + ", " + fi.ForumEnableAdvancedImage.ToString().ToLower() + "); }");
            }
            else
            {
                ucBBEditor.IsLiveSite = this.IsLiveSite;
                ucBBEditor.ShowImage = fi.ForumEnableImage;
                ucBBEditor.ShowQuote = fi.ForumEnableQuote;
                ucBBEditor.ShowBold = fi.ForumEnableFontBold;
                ucBBEditor.ShowItalic = fi.ForumEnableFontItalics;
                ucBBEditor.ShowUnderline = fi.ForumEnableFontUnderline;
                ucBBEditor.ShowStrike = fi.ForumEnableFontStrike;
                ucBBEditor.ShowCode = fi.ForumEnableCodeSnippet;
                ucBBEditor.ShowColor = fi.ForumEnableFontColor;
                ucBBEditor.ShowURL = fi.ForumEnableURL;
                ucBBEditor.ShowAdvancedImage = fi.ForumEnableAdvancedImage;
                ucBBEditor.ShowAdvancedURL = fi.ForumEnableAdvancedURL;

                // WAI validation
                lblText.AssociatedControlClientID = ucBBEditor.TextArea.ClientID;
            }

            #endregion
        }

        // Do not show subscribe checkbox if no post can't be added under the post
        if ((replyPi != null) && (replyPi.PostLevel >= ForumPostInfoProvider.MaxPostLevel - 1))
        {
            plcSubscribe.Visible = false;
        }

        #region "Resources"

        rfvEmail.ErrorMessage = ResHelper.GetString("Forums_WebInterface_ForumNewPost.emailErrorMsg");
        rfvSubject.ErrorMessage = ResHelper.GetString("Forums_WebInterface_ForumNewPost.subjectErrorMsg");
        lblText.Text = ResHelper.GetString("Forums_WebInterface_ForumNewPost.text");
        rfvText.ErrorMessage = ResHelper.GetString("Forums_WebInterface_ForumNewPost.textErrorMsg");
        rfvUserName.ErrorMessage = ResHelper.GetString("Forums_WebInterface_ForumNewPost.usernameErrorMsg");
        btnOk.Text = ResHelper.GetString("general.ok");
        btnCancel.Text = ResHelper.GetString("general.cancel");
        btnPreview.Text = ResHelper.GetString("Forums_WebInterface_ForumNewPost.Preview");
        lblSubscribe.Text = ResHelper.GetString("Forums_WebInterface_ForumNewPost.Subscription");
        lblSignature.Text = ResHelper.GetString("Forums_WebInterface_ForumNewPost.Signature");
        lblPostIsAnswerLabel.Text = ResHelper.GetString("ForumPost_Edit.PostIsAnswer");
        lblPostIsNotAnswerLabel.Text = ResHelper.GetString("ForumPost_Edit.PostIsNotAnswer");

        #endregion

        if (!this.IsLiveSite && !RequestHelper.IsPostBack() && process)
        {
            ReloadData();
        }
    }


    /// <summary>
    /// When post is added
    /// </summary>
    public event EventHandler OnInsertPost;


    /// <summary>
    /// When preview is requested
    /// </summary>
    public event EventHandler OnPreview;


    /// <summary>
    /// Btn OK handler
    /// </summary>
    protected void btnOK_Click(object sender, EventArgs e)
    {
        if (!CheckPermissions("cms.forums", CMSAdminControl.PERMISSION_MODIFY))
        {
            return;
        }

        if (editPi == null)
        {
            editPi = ForumPostInfoProvider.GetForumPostInfo(this.EditPostID);
        }

        if (editPi != null)
        {
            editPi.PostLastEdit = DateTime.Now;
            editPi.PostIsAnswer = ValidationHelper.GetInteger(txtPostIsAnswer.Text, editPi.PostIsAnswer);
            editPi.PostIsNotAnswer = ValidationHelper.GetInteger(txtPostIsNotAnswer.Text, editPi.PostIsNotAnswer);
        }
        else
        {
            // Create new post
            editPi = new ForumPostInfo();

            // Set as reply
            if (replyPi != null)
            {
                editPi.PostParentID = replyPi.PostId;
            }

            editPi.PostUserID = CMSContext.CurrentUser.UserID;
            editPi.PostForumID = fi.ForumID;
            editPi.PostTime = DateTime.Now;
            editPi.PostApproved = true;
            editPi.PostApprovedByUserID = CMSContext.CurrentUser.UserID;

        }

        #region "Security"

        string result = new Validator().NotEmpty(txtSubject.Text, rfvSubject.ErrorMessage).NotEmpty(txtUserName, rfvUserName.ErrorMessage).Result;

        // Check if is some text in TextArea or in HTMLEditor
        if (result == "")
        {
            if (fi.ForumHTMLEditor)
            {
                if (htmlTemplateBody.ResolvedValue.Trim() == "")
                {
                    result = rfvText.ErrorMessage;
                }
                editPi.PostText = htmlTemplateBody.ResolvedValue;
            }
            else
            {
                if (DiscussionMacroHelper.RemoveTags(ucBBEditor.Text).Trim() == "")
                {
                    result = rfvText.ErrorMessage;
                }
                editPi.PostText = ucBBEditor.Text;
            }
        }

        if ((fi.ForumRequireEmail || chkSubscribe.Checked || (txtEmail.Text != String.Empty)) && result == "")
        {
            result = new Validator().IsEmail(txtEmail.Text, rfvEmail.ErrorMessage).Result;
            if (!String.IsNullOrEmpty(result))
            {
                lblError.Visible = true;
                lblError.Text = result;
                if (chkSubscribe.Checked && String.IsNullOrEmpty(txtEmail.Text))
                {
                    lblError.Text = ResHelper.GetString("Forums.Emailsubscribe");
                }
                this.Visible = true;
                return;
            }
        }

        #endregion

        if (String.IsNullOrEmpty(result))
        {
            if (fi.ForumType == 0)
            {
                editPi.PostType = (radTypeQuestion.Checked) ? 1 : 0;
            }

            editPi.PostUserName = txtUserName.Text;
            editPi.PostSubject = txtSubject.Text;
            editPi.PostUserMail = txtEmail.Text;
            editPi.PostUserSignature = txtSignature.Text;

            ForumPostInfoProvider.SetForumPostInfo(editPi);
            this.EditPostID = editPi.PostId;

            #region "Subscription"

            if ((chkSubscribe.Checked) && (!String.IsNullOrEmpty(editPi.PostUserMail)) && (!ForumSubscriptionInfoProvider.IsSubscribed(editPi.PostUserMail.Trim(), editPi.PostForumID, editPi.PostId)))
            {
                ForumSubscriptionInfo fsi = new ForumSubscriptionInfo();
                fsi.SubscriptionForumID = this.ForumID;
                fsi.SubscriptionEmail = editPi.PostUserMail;
                fsi.SubscriptionPostID = editPi.PostId;
                fsi.SubscriptionUserID = CMSContext.CurrentUser.UserID;
                fsi.SubscriptionGUID = Guid.NewGuid();

                ForumSubscriptionInfoProvider.SetForumSubscriptionInfo(fsi);
            }

            #endregion

            ClearForm();

            if (OnInsertPost != null)
            {
                OnInsertPost(this, null);
            }

            this.RaiseOnSaved();

        }
        else
        {
            lblError.Visible = true;
            lblError.Text = result;
            return;
        }


    }


    public event EventHandler OnCancelClick;


    /// <summary>
    /// Cancel click handler
    /// </summary>
    protected void btnCancel_Click(object sender, EventArgs e)
    {
        ClearForm();

        if (OnCancelClick != null)
        {
            OnCancelClick(this, null);
        }

        if (RedirectEnabled)
        {
            UrlHelper.Redirect(URLRewriter.CurrentURL);
        }
    }


    /// <summary>
    /// Preview click handler
    /// </summary>
    protected void btnPreview_Click(object sender, EventArgs e)
    {
        ForumPostInfo fp = new ForumPostInfo();

        fp.PostUserMail = txtEmail.Text;
        fp.PostUserName = txtUserName.Text;
        fp.PostText = ucBBEditor.Text;

        if (fi.ForumHTMLEditor)
        {
            fp.PostText = htmlTemplateBody.ResolvedValue;
        }

        fp.PostTime = DateTime.Now;
        fp.PostSubject = txtSubject.Text;
        fp.PostUserSignature = txtSignature.Text;
        fp.PostForumID = (editPi != null) ? editPi.PostForumID : this.ForumID;
        fp.PostUserID = (editPi != null) ? editPi.PostUserID : CMSContext.CurrentUser.UserID;

        if (replyPi != null)
        {
            fp.PostLevel = replyPi.PostLevel + 1;
        }

        ForumPost1.DisplayOnly = true;
        ForumPost1.PostDataRow = fp.DataClass.DataRow;
        pnlReplyPost.Visible = true;
        lblHeader.Text = ResHelper.GetString("Forums_ForumNewPost_Header.Preview");

        // Fire the event
        if (OnPreview != null)
        {
            OnPreview(this, null);
        }

    }


    /// <summary>
    /// Clear from fields
    /// </summary>
    public override void ClearForm()
    {
        ucBBEditor.Text = "";
        txtSubject.Text = "";
        txtEmail.Text = "";
        txtUserName.Text = "";
        txtSignature.Text = "";
        htmlTemplateBody.ResolvedValue = "";
        pnlReplyPost.Visible = false;
    }


    /// <summary>
    /// Reloads the form data
    /// </summary>
    public override void ReloadData()
    {
        fi = ForumInfoProvider.GetForumInfo(this.ForumID);
        replyPi = ForumPostInfoProvider.GetForumPostInfo(this.ReplyToPostID);
        editPi = ForumPostInfoProvider.GetForumPostInfo(this.EditPostID);
        CurrentUserInfo cui = CMSContext.CurrentUser;

        // Edit post
        if (editPi != null)
        {
            ForumPost1.PostID = editPi.PostId;
            ForumPost1.DisplayOnly = true;

            //btnCancel.Enabled = false;
            ucBBEditor.Text = editPi.PostText;
            htmlTemplateBody.ResolvedValue = editPi.PostText;
            txtSubject.Text = editPi.PostSubject;
            txtSignature.Text = editPi.PostUserSignature;
            txtEmail.Text = editPi.PostUserMail;
            txtUserName.Text = editPi.PostUserName;
            plcIsAnswer.Visible = true;
            plcIsNotAnswer.Visible = true;
            txtPostIsAnswer.Text = editPi.PostIsAnswer.ToString();
            txtPostIsNotAnswer.Text = editPi.PostIsNotAnswer.ToString();

            if ((fi != null) && (fi.ForumType == 0) && (editPi.PostLevel == 0))
            {
                if (editPi.PostType == 0)
                {
                    radTypeDiscussion.Checked = true;
                }
                else
                {
                    radTypeQuestion.Checked = true;
                }
            }
            else
            {
                plcThreadType.Visible = false;
            }
        }
        else
        {
            // Reply to post
            if (replyPi != null)
            {
                pnlReplyPost.Visible = true;
                ForumPost1.PostID = replyPi.PostId;
                ForumPost1.DisplayOnly = true;

                plcThreadType.Visible = false;
                txtSignature.Text = cui.UserSignature;
                txtEmail.Text = cui.Email;

                if (!String.IsNullOrEmpty(cui.UserNickName))
                {
                    txtUserName.Text = cui.UserNickName;
                }
                else
                {
                    txtUserName.Text = cui.UserName;
                }

                if (!replyPi.PostSubject.StartsWith(ResHelper.GetString("Forums.ReplyPrefix")))
                {
                    txtSubject.Text = ResHelper.GetString("Forums.ReplyPrefix") + replyPi.PostSubject;
                }
                else
                {
                    txtSubject.Text = replyPi.PostSubject;
                }
                btnCancel.Enabled = true;
            }
            // Create new thread
            else
            {
                btnCancel.Enabled = false;
                txtSignature.Text = cui.UserSignature;
                txtEmail.Text = cui.Email;

                if (!String.IsNullOrEmpty(cui.UserNickName))
                {
                    txtUserName.Text = cui.UserNickName;
                }
                else
                {
                    txtUserName.Text = cui.UserName;
                }
            }
        }
    }
}
